@@ -149,7 +149,7 @@ panel.superpose.dl <- structure(function
 
   ## Exploring custom panel and panel.groups functions
   library(ggplot2)
-  data(BodyWeight,package="nlme")
+  library(nlme)
   ## Say we want to use a simple linear model to explain rat body weight:
   fit <- lm(weight~Time+Diet+Rat,BodyWeight)
   bw <- fortify(fit,BodyWeight)
